﻿/*
===============================================================================
Source   : https://resource.dopus.com/t/fast-search-with-everything-v2/31470
AUTHOR   : Aussie & Ken
Function : Find items via Everything
Modified : 2023-03-18
Version  : v2.4.9k
Change   :
2022-03-18 Reduce response time
2022-12-20 Find by Tags List
2022-11-26 Judge the result
2022-11-22 Add search history
2022-08-14 Save each search with time
===============================================================================
*/



// History

// 20190205 v2.0 Front end to everything.exe and es.exe
// 20190205 v2.0 Replacement for $es. See https://resource.dopus.com/t/fast-search-with-everything/21974 
// 20190205 v2.0 Incorporates ET, ES and CLIPTOCOL commands into a single wrapper
// 20190206 v2.1 Use clipboard for ES searches in order to deal with non-ASCII results
// 20190206 v2.1 because ES output redirected to a file does not handle non-ASCII file names properly
// 20190207 v2.1a Revert to v2.0 base
// 20190207 v2.0a Use UTF8 flag on import of text file exported by ES which us UTF8 without a BOM
// 20190207 v2.0a See https://www.voidtools.com/forum/viewtopic.php?f=5&t=7473
// 20190207 v2.0a Add matchdiacritics configuration option
// 20190209 v2.2 Add esplus and cliptocolplus commands to append to target
// 20190221 v2.3 Fix handling of HERE button to avoid adding double backslash to root paths
// 20190222 v2.4 Add config option for choice of Anywhere, Below, Here as default

//-- Add string trim() method

String.prototype.trim = function(s){s=s||"\\s";return this.replace(new RegExp("^("+s+"){1,}\|("+s+"){1,}$","g"),"");}

//-- Global Variables

//Set time to global variable
  var Time = DOpus.Create.Date.Format("D#yyMMdd-T#HHmmss");
  DOpus.Vars.Set("Time") = Time;
  var Time = DOpus.Vars.Get("Time");



var me = {
    //name:"ets",
	name:Time,                                                  //Collection name changed to current time
	version:"2.4.9k (March 18nd, 2023)",
	desc:"Front end to everything.exe and es.exe",
	copyright:"(c) 2019 Aussie",
	template:"",
	minver:"12",
	et:"et",
	es:"es",
	esplus:"esplus",
	cliptocol:"cliptocol",
	cliptocolplus:"cliptocolplus",
	cmdname:""
}
var d, dbg, dlg, fsu;
var colname = me.name;
//var colspec = "coll://"+"Everything\\"+colname;               //Change the collection path name
var colcmd = "/home\\dopusrt.exe /col ";
var colimp = colcmd+"import /create /utf8 ";
var etspec, etargs = " -newwindow -s ";
var esspec, esargs = " -dm -size -full-path-and-name -sort-date-modified -export-txt ";
var tmp, ForReading = 1, ForWriting = 2, ForAppending = 8;
var ms = 50; // Delay in millisecs
var waitcount = 12; // Max number of times to apply the delay
var doprompt;

function OnInit(initData)
{
	//initData.name = me.name;
	initData.name = "Everything";                                //Name to Everything
	initData.version = me.version;
	initData.desc = me.desc;
	initData.copyright = me.copyright;
	initData.default_enable = true;
	
	var cmd1 = initData.AddCommand();
	cmd1.name = me.et;
	cmd1.method = "et";
	cmd1.desc = "Simple front end to everything.exe";
	
	var cmd2 = initData.AddCommand();
	cmd2.name = me.es;
	cmd2.method = "es";
	cmd2.desc = "Create an Opus collection using ES (Everything Search) output";
	cmd2.template = "SearchString/R";
	
	var cmd3 = initData.AddCommand();
	cmd3.name = me.esplus;
	cmd3.method = "esplus";
	cmd3.desc = "Append to an Opus collection using ES (Everything Search) output";
	
	var cmd4 = initData.AddCommand();
	cmd4.name = me.cliptocol;
	cmd4.method = "cliptocol";
	cmd4.desc = "Create an Opus collection from filespecs on the clipboard";
	cmd4.template = "SearchString/R";
	
	var cmd5 = initData.AddCommand();
	cmd5.name = me.cliptocolplus;
	cmd5.method = "cliptocolplus";
	cmd5.desc = "Append to an Opus collection from filespecs on the clipboard";
    
	var cmd6 = initData.AddCommand();
	cmd6.name = "fbtl";
	cmd6.method = "Onfbtl";
	cmd6.desc = "Find by Tags List";
	
	initData.config_desc = DOpus.create.map();
	initData.config.dbg = false;
	initData.config_desc("dbg") = 'Control whether trace information is writen to "Other" log.';
	initData.config.lookin = DOpus.create.vector("/programfiles\\everything","/programfilesx86\\everything");
	initData.config_desc("lookin") = "Where to look for everything.exe and es.exe.";
	initData.config.colname = colname;
	initData.config_desc("colname") = 'Target collection name.';
	initData.config.esprompt = true;
	initData.config_desc("esprompt") = 'True = prompt if no args and no qualifier, False = the reverse.';
	initData.config.etprompt = false;
	initData.config_desc("etprompt") = 'True = prompt if no args and no qualifier, False = the reverse.';
	initData.config.deflookfor = "dm:last30secs";
	initData.config_desc("deflookfor") = "Default search term if none provided";
	initData.config.matchdiacritics = false;
	initData.config_desc("matchdiacritics") = "Control exact matching of diacritical characters.";
	initData.config.defbutton = 1;
	initData.config_desc("defbutton") = "Default button number when Below (2) and Here (3) are available .";
}

//-- Main command code

function cliptocol(ScriptCommandData, append){
	searchString = false;
	var searchString = ScriptCommandData.func.args.got_arg.SearchString ? ScriptCommandData.func.args.SearchString : "";
	
	me.cmdname = (append==true) ? me.cliptocolplus : me.cliptocol;
	if (!commoninit(ScriptCommandData)) return;
	var cliptext = DOpus.getclip("text");
	var a = cliptext.split(/\r\n/g);
	
	if(searchString != "clip") {      // Used to create a collection from file specs captured on the clipboard
	   if (fsu.newpath(a[0]).drive==0){
		   dlg.request("No valid text found on the clipboard","OK","[cliptocol]");
		   return;
	   }
	   if (!commoninit(ScriptCommandData, me.cliptocol)) return;
       var ST = DOpus.Create.StringTools;
       var FSU = DOpus.FSUtil;
       var oFileWrite = FSU.OpenFile (tmp, "wa");
       var utf8blob = ST.Encode(cliptext, "utf-8 bom");
       if (oFileWrite.error == 0)
       {
           oFileWrite.Write (utf8blob)                   // Create a temp file from clipboard text (if there is any)
       }
       oFileWrite.Close ()
    } else {      // Otherwise, search using the contents of the clipboard
	   var cmdstr = '"'+esspec+'" '+maybe_quote(a[0])+" "+esargs+tmp; // Build an es command
       var WshShell = new ActiveXObject("WScript.Shell");
       var result = WshShell.Run(String(cmdstr),0,true); // true == wait for process to finish
	   if (!waitforit(tmp)) return;   // Wait for result
	}

//Time
    var Time = DOpus.Vars.Get("Time");
	
	//if (append!=true) exec(colimp+"/clear "+maybe_quote("Everything\\"+Time)+" "+tmp); // Close     //The collection is stored in the Everything folder //else
	exec(colimp+"Everything\\"+Time+" "+tmp); // Append                                               //The collection is stored in the Everything folder
	//exec('Go "'+colspec+'" NEWTAB=findexisting');
	var colspec = "coll://"+"Everything\\"+Time;                                                      //Change the collection path name
    
	exec('Go "'+colspec+'" NEWTAB');                                                                  //Open collection in a new TAB
	commonterm();
}

function cliptocolplus(ScriptCommandData){
	cliptocol(ScriptCommandData, true);
}

function es(ScriptCommandData, append){
	me.cmdname = (append==true) ? me.esplus : me.es;
	if (!commoninit(ScriptCommandData)) return;
	var argstring = maybeprompt(me.argstring);
	if (dbg) DOpus.output("es argstring = " + argstring);
	if (argstring=="") return;
	argstring = resolvealiases(argstring);
	argstring = argstring.replace(/\|/g,'^|');
	argstring = argstring.replace(/\</g,'^<');
	argstring = argstring.replace(/\>/g,'^>');
	//var cmdstr = 'cmd.exe /s /c ""'+esspec+'" '+argstring+esargs+tmp+'"'; // Build an es command
	var cmdstr = '"'+esspec+'" '+argstring+esargs+tmp; // Build an es command
	
	//exec(cmdstr,true); // Create a temp text file containing es search results
    var WshShell = new ActiveXObject("WScript.Shell");
    var result = WshShell.Run(String(cmdstr),0,true); // true == wait for process to finish
	if (!waitforit(tmp)) return;   // Wait for result

//Time
    var Time = DOpus.Vars.Get("Time");
	
	//if (append!=true) exec(colimp+"/clear "+maybe_quote("Everything\\"+Time)+" "+tmp); // Close     //The collection is stored in the Everything folder //else
	exec(colimp+"Everything\\"+Time+" "+tmp); // Append                                               //The collection is stored in the Everything folder
	//exec('Go "'+colspec+'" NEWTAB=findexisting');
    var colspec = "coll://"+"Everything\\"+Time;                                                      //Change the collection path name
	
	exec('Go "'+colspec+'" NEWTAB');                                                                  //Open collection in a new TAB
	commonterm();
}

function esplus(ScriptCommandData) {
	es(ScriptCommandData, true);
}

function et(ScriptCommandData) {
	me.cmdname = me.et;
	if (!commoninit(ScriptCommandData)) return;
	var argstring = maybeprompt(me.argstring);
	if (argstring=="") return;
	if (dbg) DOpus.output("et argstring = " + argstring);
	argstring = resolvealiases(argstring); // e.g. /programfiles ==> "C:\Program Files"
	argstring = argstring.replace(/"/g, '"""'); // e.g. ==>  """C:\Program Files"""
	var cmdstr = 'cmd.exe /s /c ""'+etspec+'"'+etargs+' "'+argstring+'"';
	exec(cmdstr,true);
	commonterm();
}



 var cmd = DOpus.Create.Command();
 // Create folder if does not exist
 if (!DOpus.FSUtil.Exists("%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\")) {
	cmd.Runcommand('CreateFolder "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\"');
 }



function Onfbtl(ScriptCommandData)
{

    var dlg = ScriptCommandData.func.Dlg();
    var tab = ScriptCommandData.func.sourcetab;
    var cmd = ScriptCommandData.func.command;

    if (!DOpus.version.AtLeast("12.30.1"))
    {
	    dlg.request("This script requires 12.30.1 and above!", "OK");
		return
    }

    var strName;

    dlg.title = "Directory Opus";
    dlg.template = "dlgList";
    dlg.detach = true;
    dlg.Create();
	dlg.Control("static").style = "b";
	dlg.Control("static").label = "Find by Tags";                           // Topic
	dlg.Control("static2").label = "Tick the tags that shall be matched via Everyting tool.";   // Notes
	dlg.Control("static3").label = Script.LoadImage("Everything24.ico");    // Icon
	dlg.Control("static4").label = Script.LoadImage("Find.png");            // Icon
	//dlg.Control("static5").label = "D:\\Icons\\Clear.png";                  // Clear

    updateList(dlg, strName);

    dlg.Show();

	var lst = dlg.Control("list");
	var col = lst.columns;



    while (true)
	{
        var msg = dlg.GetMsg();
        if (!msg.result) break;

// Result
        if (msg.event == "focus" || dlg.Control("btnOK").focus == true)
		{
		   var tags = "";
           for (i=0; i < lst.count; i ++)
		   {
                if (lst.GetItemAt(i).checked == 1)
				{
				    tags += "tag:";
					tags += lst.GetItemAt(i).name;
					tags += "|";
                }
           }
		}

// Right click list menu
        if (msg.event == "rclick" && lst.focus == true)
		{

            var dlgMenu = DOpus.Dlg;
            if (lst.value.index > -1)
			{
				dlgMenu.choices = DOpus.Create.Vector("New", "Edit", "Delete", "Clear Filter", "Refresh", "Copy", "Cancel");
				dlgMenu.menu = DOpus.Create.Vector(1, 0, 0, 0, 0, 0, 1);
            } else {
				dlgMenu.choices = DOpus.Create.Vector("OK", "Cancel");
                dlgMenu.menu = DOpus.Create.Vector(1, 1);
            }
            var menuReturn = dlgMenu.Show;

            if (menuReturn == 1) {
				NewTags();
				updateList(dlg, strName);
            }

            if (menuReturn == 2) {
				var editTag = lst.GetItemAt(lst.value.index).name;
				tab.vars.Set('editTag', editTag);
				EditTag();
				updateList(dlg, strName);
            }

            if (menuReturn == 3) {
				var deleteTag = lst.GetItemAt(lst.value.index).name;
				tab.vars.Set('deleteTag', deleteTag);
				DeleteTag();
				updateList(dlg, strName);
            }

            if (menuReturn == 4) {
                dlg.Control("editFilter").value = "";
			    updateList(dlg, strName);
            }

            if (menuReturn == 5) {
				updateList(dlg, strName);
            }

            if (menuReturn == 6) {
                var item = lst.GetItemAt(lst.value.index).name;
				DOpus.SetClip(item);
            }
        }

// Filter
		if (msg.event == "editchange" || msg.event == "click")
		{
			updateList(dlg, strName);
		}

    }


// Add the checked tags and restore the timestamp
	if (dlg.result == 1 && tags)
	{
	    tags = tags.substr(0, tags.length-1);
	    //me.cmdname = (append==true) ? me.esplus : me.es;
	    if (!commoninit(ScriptCommandData)) return;
/*
	    var argstring = maybeprompt(me.argstring);
	    if (dbg) DOpus.output("es argstring = " + argstring);
	    if (argstring=="") return;
		argstring = resolvealiases(argstring);
*/
	    var argstring = resolvealiases(tags);
	    argstring = argstring.replace(/\|/g,'^|');
	    argstring = argstring.replace(/\</g,'^<');
	    argstring = argstring.replace(/\>/g,'^>');
		
	    //var cmdstr = 'cmd.exe /s /c ""'+esspec+'" '+argstring+esargs+tmp+'"'; // Build an es command
		//exec(cmdstr,true); // Create a temp text file containing es search results
		var cmdstr = '"'+esspec+'" '+argstring+esargs+tmp; // Build an es command
        var WshShell = new ActiveXObject("WScript.Shell");
        var result = WshShell.Run(String(cmdstr),0,true); // true == wait for process to finish
	    if (!waitforit(tmp)) return;   // Wait for result

//Time
        var Time = DOpus.Vars.Get("Time");
	
	    //if (append!=true) exec(colimp+"/clear "+maybe_quote("Everything\\"+Time)+" "+tmp); // Close     //The collection is stored in the Everything folder //else
	    exec(colimp+"Everything\\"+Time+" "+tmp); // Append                                               //The collection is stored in the Everything folder
	    //exec('Go "'+colspec+'" NEWTAB=findexisting');
        var colspec = "coll://"+"Everything\\"+Time;                                                      //Change the collection path name
	
	    exec('Go "'+colspec+'" NEWTAB');                                                                  //Open collection in a new TAB
	    commonterm();
	}
}



//-- Functions called from main script code

function commoninit(ScriptCommandData) {
	
	d = ScriptCommandData;
	dlg = d.func.dlg;
	fsu = DOpus.FSUtil;
	
	// Debug ?
	dbg = Script.config.dbg;
	
	// Who am I?
	if (dbg) DOpus.output(me.name+" ("+me.cmdname+") version " + me.version + " starting..");
	
	// Check minimum version requirements
	if (!DOpus.version.AtLeast(me.minver)){
		dlg.request(me.name+" requires DOpus version " + me.minver + " or later","OK","["+me.name+"]");
		return false;
	}
	
	// Process args
	// Force a space before | to deal with (e.g.) path:/alias1|path:/alias2
	me.argstring = d.cmdline.slice(me.cmdname.length).trim().replace(/\|/g,' |');
	if (dbg) DOpus.output("me.argstring = "+me.argstring);
	if (Script.config.matchdiacritics) esargs = " -a"+esargs;
	
	// Initialise variables
	colname = Script.config.colname;
	doprompt = (me.cmdname==me.et) ? Script.config.etprompt : Script.config.esprompt;	
	
	// Check for everything.exe availability
	etspec = findexe("everything");
	if (etspec==""){		
		dlg.request("everything.exe not found","OK");
		return false;
	}
	if (dbg) DOpus.output("etspec = " + etspec);
	
	// Make sure everything.exe is running
	var str = "everything.exe";
	if (DOpus.Create.Sysinfo.FindProcess(str)==0) exec(etspec+" -first-instance -startup");
	if (me.cmdname==me.et) return true; // No need to check for es.exe
	
	// Check for es.exe availability
	esspec = findexe("es");
	if (esspec==""){		
		dlg.request("es.exe not found","OK");
		return false;
	}
	if (dbg) DOpus.output("esspec = " + esspec);
	
	// Define target file for es.exe or text ex the clipboard
	tmp = maybe_quote(fsu.resolve("/temp\\"+colname+".out"));
	// Erase any existing temp file
	//exec("Delete QUIET NORECYCLE FILE="+tmp);      //↓Don't waste time
	
	return true;
}

function commonterm(){
	if (dbg) DOpus.output(me.name+" ("+me.cmdname+") version " + me.version + " exiting normally..");
	cmd.Runcommand("Delete QUIET NORECYCLE FILE="+tmp)      //↑
}

function exec(cmdstr,hide){
	//var cmd = DOpus.Create.Command;
	if (dbg) DOpus.output("cmdstr = " + cmdstr);
	if (hide) cmd.addline("@runmode:hide");
	cmd.addline(cmdstr);
	cmd.run;
}

function findexe(name){	
	// Attempt to locate an executable (e.g. findexe("es") to locate es.exe)
	var fn = String(name).trim()+".exe";
	var path =  (DOpus.vars.exists("xx-"+fn)) ? DOpus.vars.get("xx-"+fn) : "";
	if (fsu.exists(path+"/"+fn)) return path+"/"+fn;
	var lookin =  (typeof Script.config.lookin =="undefined") ? DOpus.create.vector() : Script.config.lookin;
	for (var i = 0; i < lookin.count; i++){
		path = fsu.resolve(lookin(i));
		if (fsu.exists(path+"/"+fn)){
			DOpus.vars.set("xx-"+fn, String(path));
			DOpus.vars("xx-"+fn).persist = true;
			return path+"/"+fn;
		}
	}
	dlg.request(fn+" not found","OK","[Everything]");
	return "";	
}

 var fsu = DOpus.FSUtil;
 var ST = DOpus.Create.StringTools();

function maybeprompt(str){
	eState = false;
	doprompt = (d.func.qualifiers=="none") ? doprompt : !doprompt;
	var promptstr = String(str).trim();
	if ((doprompt==false) && (promptstr!="")) return promptstr;
	if(promptstr=="cf") {eState=true; promptstr=""}
	if (promptstr==""){
		if (d.func.sourcetab.selected.count==1){ // Set default search term based on first selected item, if possible
			//promptstr = (d.func.sourcetab.selected(0).ext=="") ? "" : ".*";
			promptstr = (d.func.sourcetab.selected(0).ext=="") ? "" : "";                            //Only item's basename
			//promptstr = "wfn:"+maybe_quote(d.func.sourcetab.selected(0).name_stem_m+promptstr);
			promptstr = maybe_quote(d.func.sourcetab.selected(0).name_stem_m);                       //Remove "wfn:"
		}
/*
		else{
			promptstr = Script.config.deflookfor;
			if (promptstr=="") promptstr = "dm:last20secs";
		}
*/
	}
	var path = String(d.func.sourcetab.path);
	if (dbg) DOpus.output("path = " + path);
	var hereok = (path.toLowerCase().indexOf("coll:")!=0);
	hereok = (hereok) && (path.toLowerCase().indexOf("::")!=0);
	hereok = (hereok) && (path.toLowerCase().indexOf("\\\\localhost")!=0);
/*
	dlg.title = "[Everything]";
	dlg.message = "Enter search term(s)";
	dlg.defvalue = promptstr;
	dlg.max = 0;
	dlg.select = true;
	dlg.buttons = "Anywhere|";
	if (hereok){
		dlg.buttons = dlg.buttons+"Below|Here|";
		var defbutton = Script.config.defbutton;
		if ((defbutton==2) || (defbutton==3)) dlg.defid = defbutton;
	}
	dlg.buttons = dlg.buttons+"Cancel";
	dlg.show;
	if (dlg.result==0) return "";
	// Force a space before | to deal with (e.g.) path:/alias1|path:/alias2
	var fixstr = dlg.input.trim().replace(/\|/g,' |');
*/

//Read the history of the EverythingForDopus_History.ini file
  var filepath = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\EverythingForDopus_History.ini";   //File path

  if(fsu.Exists(filepath)) {
     var bom = fsu.OpenFile(filepath).Read();
  
     textbom = ST.Decode(bom, "utf-8 bom");
     } else {
     var textbom = "";
     }
    
    var dlg = DOpus.Dlg;
    
    dlg.title = "Directory Opus";                                           // Tittle
    dlg.template = "dlgCombox";
	dlg.detach = true;
    dlg.Create();
	dlg.Control("static").style = "b";
	dlg.Control("static").label = "[Everything]";                           // Topic
	dlg.Control("static2").label = "Enter search term(s)";                  // Notes
	dlg.Control("static3").label = Script.LoadImage("Everything24.ico");    // Icon
	
	if(eState) {
	   dlg.Control("btn3").enabled = false;         // 使按钮不可见
	   dlg.Control("btn3").visible = false;
	   dlg.Control("btn4").enabled = false;
	   dlg.Control("btn4").visible = false;
	}
	
    var myArray = textbom.split("\r\n");            //Seprator
	
	if(promptstr != "") {
	   dlg.Control("combo").label = promptstr;      //Default value
	  } else {
	  dlg.Control("combo").label = myArray[0];
	  }
	
	dlg.Control("combo").SelectRange(0, -1);        //Select Default value
    for(i = 0; myArray[i]; i++) {
		dlg.Control("combo").AddItem(myArray[i]);   //Add each item to drop-down list
	    }
    
    dlg.Show();

//msg loop
	do
	{
		msg = dlg.GetMsg();
/*
        //Selection
		if(msg == true && dlg.Control("combo").focus == true) {
		   DOpus.Output(dlg.Control("combo").label);
		}
*/
        //Result
		if(dlg.result == 1) {
		   var fixstr = dlg.Control("combo").label;
		   fixstr = fixstr.trim().replace(/\|/g,' |');
		} else {
		if(dlg.result == 0) {
		   var fixstr = "";
		   return "";
		  }
		}
		
		//If the button 3 is pressed
		if(dlg.result == 2) {
		   var fixstr = dlg.Control("combo").label;
		   fixstr = fixstr.trim().replace(/\|/g,' |');
		}
		
		//If the button 4 is pressed
		if(dlg.result == 3) {
		   var fixstr = dlg.Control("combo").label;
		   fixstr = fixstr.trim().replace(/\|/g,' |');
		}
	
	} while (msg == true);


//Create a file to save the history
  if(fixstr != "") {
	var newHistory = "";
    for(i = 0; myArray[i]; i++) {
	    if(i == 15) {break};
        if(fixstr != myArray[i]) {newHistory += "\r\n" + myArray[i]};
	   }
    newHistory = fixstr + newHistory;
    
    var filepath = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\EverythingForDopus_History.ini";  //File path
    var oFileWrite = fsu.OpenFile (filepath, "wa");
    var utf8blob = ST.Encode(newHistory, "utf-8 bom");
	
    if(oFileWrite.error == 0)
	  {
       oFileWrite.Write (utf8blob)
      }
    oFileWrite.Close ()
    }
    
	if (dlg.result==3) return fixstr+" parent:"+maybe_quote(path);
	if (dlg.result==2 || eState) return fixstr+" path:"+maybe_quote(path);
	else return fixstr;
}

function maybe_quote(str){
	// Avoid unnecessary use of quotes and always force to string type
	return (String(str).indexOf(" ")<0) ? String(str) : '"'+String(str)+'"';
}

function resolvealiases(str){
	// Resolve alias(es) (e.g. /temp, /scripts) and always force to string type
	var resolved = String(str);
	var n = resolved.indexOf("/");
	if (n<0) return resolved;
	
	var pfx = resolved.slice(0,n);
	var alias = resolved.slice(n);
	var i = alias.indexOf(" ");
	if (i<0) i = alias.indexOf('\\');
	if (i<0) i = alias.indexOf('"');
	if (i<0) var sfx = "";
	else{
		var sfx = alias.slice(i);
		alias = alias.slice(0,i);
	}
	// See https://resource.dopus.com/t/temp-alias-produces-short-form-name/31304
	var path = fsu.newPath(fsu.resolve(alias)).longpath;
	path = pfx+path+sfx;
	// Keep resolving until all aliases are processed
	return (path==resolved) ? resolved : resolvealiases(path);
}

function waitforit(fn){
	// Wait for a new file (fn) to be completely written
	var i = 0;
	while (i++<waitcount){
		if (fsu.exists(fn)) break;
		if (dbg) DOpus.output("i = "+i+" : "+fn);
		DOpus.delay(ms);
	}
	if (!fsu.exists(fn)) return false;
	var i = 0;
	var size0 = 0;
	var size1 = Number(fsu.getitem(fn).size);
	while (i++<waitcount){
		if (dbg) DOpus.output("i = "+i+" : size0 = "+size0+" size1 = "+size1);
		if (size1!=size0) size0 = size1;
		else{
			DOpus.delay(ms);
			size1 = Number(fsu.getitem(fn).size);
			if (size1==size0 && size1 != 0) return true;
			else if (size1==0 && i==2) dlg.request("[Everything]\r\n\r\nNothing found!","OK");      //If no result
		}
	}
	return false;
}



 var tab = DOpus.listers.lastactive.activetab;



function updateList(dlg, strName) {

    var lst = dlg.Control("list");
	var col = lst.columns;

// Add columns
	var vecOrder = col.GetDisplayOrder();
    if (vecOrder.size < 1)
	{
	    col.AddColumn("0:Name");
	    col.AddColumn("1:Index");
	}

    lst.redraw = false;
    lst.RemoveItem(-1);

// List items
    //var text = "tag1, tag2, tag3, tag4, tag5, tag6, tag7, tag8, tag9, tag10, tag11, tag12, tag13, tag14, tag15, tag16, tag17, tag18, tag19, tag20, tag21, tag22, tag23, tag24, tag25, tag26, tag27, tag28, tag29, tag30, tag31, tag32, tag33, tag34, tag35";
	Checktag();
	var text = tab.vars.Get('DOpusTag');
	var myArray = text.split(", ");                           // Seprator

	var idx;
	var count = 0;
	var ii = 0;

    // Get var ADScheck
    var ADS = tab.vars.Get('ADScheck');

// Add checked list
    if (ADS != "")
	{
	    var array = ADS.split(", ");                          // Seprator
		for (i = 0; array[i]; i++)
		{
		     count++;
			 var checkedTag = array[i];

	         var strFilter = dlg.Control("editFilter").value;   // Get filter value
	         strFilter = strFilter.toLowerCase();

		     if (checkedTag.search(strFilter) == -1) {
			     continue;
		     }
	         idx = lst.AddItem(array[i]);                     // Add each item to list
             lst.GetItemAt(idx).subitems(0) = count;          // Add number to column 1 (Index)
			 lst.GetItemAt(idx).checked = 1;

             ii++;
			 var checkedCase = 1;
			 checkedTag = "";
			 strFilter = "";
	    }
    }

// Add list
    var tagCase;
    for (i = 0; myArray[i]; i++)
	{
        strName = myArray[i];
		var strRealName = myArray[i];

        // Remove Duplicate
        tagCase = "";
		if (checkedCase == 1)
		{
			for (ii = 0; array[ii]; ii++)
			{
				if (array[ii] == strName)
				{
					tagCase	= 1;
				}
			}
		}

		if (tagCase == "")
		{
			count++;
	        var strFilter = dlg.Control("editFilter").value;      // Get filter value
	        strFilter = strFilter.toLowerCase();

		    if (strRealName.search(strFilter) == -1) {
			    continue;
		    }

            idx = lst.AddItem(strName);                           // Add each item to list
		    lst.GetItemAt(idx).subitems(0) = count;               // Add number to column 1 (Index)

            ii++;
		    strRealName = "";
		    strFilter = "";
		}
    }

// Set the columns display order
	if (true)
	{
	    col.SetDisplayOrder(1,0);
	}

// Resize columns based on number of lists
    //col.AutoSize();
	col.GetColumnAt(1).width = String(i).length+25;
	if (ii > 27)
	{
	   col.GetColumnAt(0).width = 420-String(i).length;
	}
	else
	{
	   col.GetColumnAt(0).width = 437-String(i).length;
	}
	lst.redraw = true;
}



function Checktag()
{

    var eItem, ADSfile, text, textbom, myArray, ADS2, ADS = "", i = 0;

	if (tab.selected.count > 0)
	{
        for (var eSel = new Enumerator(tab.selected); !eSel.atEnd(); eSel.moveNext())
        {
	         i++;
	         ADS2 = ADS;
	         eItem = eSel.item();
             ADSfile =  eItem + ":Tags";
             // If ADSfile exists
             if (fsu.Exists(ADSfile))
             {
                 text = fsu.OpenFile(ADSfile).Read();          // Read file
                 textbom = ST.Decode(text, "utf-8 bom");       // Decode as UTF-8 with BOM
                 myArray = textbom.split("\r\n");              // The seprator is \r\n
                 ADS = myArray[0];                             // Get the first line
             }
             else {ADS = ""; break}
	         if (i > 1 && ADS != ADS2) {ADS = ""; break};
        }
    }
	ADS = format_tags(ADS);
    tab.vars.Set('ADScheck', ADS);

// Read the tag file DOpusTag.dat and set it to variable DOpusTag
    var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
    if (DOpus.FSUtil.Exists(tagFile))
    {
        var tags = fsu.OpenFile(tagFile).Read();
        var tags = ST.Decode(tags, "utf-8 bom");
    } else {var tags = "";}
	tags = format_tags(tags);
    tab.vars.Set('DOpusTag', tags);
}



function NewTags() {

    var dlgEdit = DOpus.Dlg;
    dlgEdit.title = "Directory Opus";
    dlgEdit.template = "dlgEditTags";
    dlgEdit.detach = true;
    dlgEdit.Create();

	dlgEdit.Control("static").style = "b";
	dlgEdit.Control("static").label = 'Enter New Tags';                         // Topic
	dlgEdit.Control("static2").label = 'One tag per line, or enter new tags separated by commas.';   // Notes
	dlgEdit.Control("static3").label = Script.LoadImage("Edit-new.png");        // Icon

	dlgEdit.Control("editTags").value = "";                                     // Default value

    dlgEdit.Show();

	var newTags, newTags2 = "", tagsTMP = "|", tagCase, dlgValueOld, dlgValue = "";

// msg loop
	do
	{
	   dlgValueOld = dlgEdit.Control("editTags").value;
	   msg = dlgEdit.GetMsg();

	   if (msg == true) {dlgValue = dlgEdit.Control("editTags").value};

	   if (msg.qualifiers == "none") {tagsTMP = dlgEdit.Control("editTags").value};

	   var dlgValueArray = dlgValue.split("\r");
	   var newDlgValue = "";
	   for (i = 0; dlgValueArray[i]; i++)
	   {
		   newDlgValue += dlgValueArray[i].trim();
	   }

	   var OldArray = dlgValueOld.split("\r");
	   var dlgValueOld2 = "";
	   for (i = 0;OldArray[i]; i++)
	   {
		   dlgValueOld2 += OldArray[i].trim()
	   }

       // Result
	   if (dlgEdit.result == 1 || msg.qualifiers == "ctrl" && msg.event == "editchange" && newDlgValue == dlgValueOld2 && dlgValue != DOpus.GetClip || msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValue.trim() == tagsTMP.trim() || msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValueArray[0].trim() == "" && dlgValue.trim() != "")
	   {
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValueArray[0].trim() == "" && dlgValue.trim() != "")
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("0")
			   newTags = dlgValue
		   }
		   else
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValue.trim() == tagsTMP.trim())
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("1")
			   newTags = dlgValue
		   }
		   else
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && newDlgValue == dlgValueOld2 && dlgValue != DOpus.GetClip)
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("2")
			   newTags = dlgValueOld
		   }
		   else
		   {
			   newTags = dlgEdit.Control("editTags").value
			   //DOpus.Output("3")
		   }

		   if (newTags.trim() == "") {return};
		   newTags = format_tags(newTags);
		   newTags = remove_duplicate(newTags);

		   var tags = tab.vars.Get('DOpusTag');
		   var tagsArray = tags.split(", ");                            // Seprator

           var array = newTags.split(", ");
           // Remove Duplicate
		   for (i = 0; array[i]; i++)
		   {
			    tagCase = "";
		        for (ii = 0; tagsArray[ii]; ii++)
				{
				     if (array[i] == tagsArray[ii])
					 {
						 tagCase = 1;
					 }
				}

			    if (tagCase != 1)
				{
					newTags2 += array[i] + ", ";
				}
           }

           if (newTags2 == "") return;
		   var newTags3 = newTags2 + tags;
           tab.vars.Set('DOpusTag', newTags3);

           // Write new tags to DOpusTag.dat
		   var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
		   var oFileWrite = fsu.OpenFile(tagFile, "wa");
           var utf8bom = ST.Encode(newTags3	, "utf-8 bom");   // Decode as UTF-8 with BOM
           if (oFileWrite.error == 0)
           {
               oFileWrite.Write(utf8bom)
           }
              oFileWrite.Close()
	   }
	} while (msg == true)
}



function EditTag() {

    var dlgEdit = DOpus.Dlg;
    dlgEdit.title = "Directory Opus";
    dlgEdit.template = "dlgEditTags";
    dlgEdit.detach = true;
    dlgEdit.Create();

	dlgEdit.Control("static").style = "b";
	dlgEdit.Control("static").label = 'Edit Tags';                              // Topic
	dlgEdit.Control("static2").label = 'One tag per line, or enter new tags separated by commas.';   // Notes
	dlgEdit.Control("static3").label = Script.LoadImage("Edit.png");            // Icon

	var editTag = tab.vars.Get('editTag', editTag);
	dlgEdit.Control("editTags").value = editTag;                                // Default value

    dlgEdit.Show();

	var newTag, tagsTMP = "|", tagCase, dlgValueOld, dlgValue = "";

// msg loop
	do
	{
	   dlgValueOld = dlgEdit.Control("editTags").value;
	   msg = dlgEdit.GetMsg();

	   if (msg == true) {dlgValue = dlgEdit.Control("editTags").value};

	   if (msg.qualifiers == "none") {tagsTMP = dlgEdit.Control("editTags").value};

	   var dlgValueArray = dlgValue.split("\r");
	   var newDlgValue = "";
	   for (i = 0; dlgValueArray[i]; i++)
	   {
		   newDlgValue += dlgValueArray[i].trim();
	   }

	   var OldArray = dlgValueOld.split("\r");
	   var dlgValueOld2 = "";
	   for (i = 0;OldArray[i]; i++)
	   {
		   dlgValueOld2 += OldArray[i].trim()
	   }

       // Result
	   if (dlgEdit.result == 1 || msg.qualifiers == "ctrl" && msg.event == "editchange" && newDlgValue == dlgValueOld2 && dlgValue != DOpus.GetClip || msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValue.trim() == tagsTMP.trim() || msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValueArray[0].trim() == "" && dlgValue.trim() != "")
	   {
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValueArray[0].trim() == "" && dlgValue.trim() != "")
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("0")
			   newTag = dlgValue
		   }
		   else
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && dlgValue.trim() == tagsTMP.trim())
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("1")
			   newTag = dlgValue
		   }
		   else
		   if (msg.qualifiers == "ctrl" && msg.event == "editchange" && newDlgValue == dlgValueOld2 && dlgValue != DOpus.GetClip)
		   {
			   dlgEdit.EndDlg;
			   //DOpus.Output("2")
			   newTag = dlgValueOld
		   }
		   else
		   {
			   newTag = dlgEdit.Control("editTags").value
			   //DOpus.Output("3")
		   }

		   if (newTag.trim() == "") {return};
		   newTag = format_tags(newTag);
		   newTag = remove_duplicate(newTag);

		   // Remove duplicate
	       var tags = tab.vars.Get('DOpusTag');
		   var tagsArray = newTag.split(", ");
		   var tagsArray2 = tags.split(", ");                            // Seprator
		   var tagCase, newTags = "";
		   for (i = 0; tagsArray2[i]; i++)
		   {
			    tagCase = "";
			    for (ii = 0; tagsArray[ii]; ii++)
				{
				     if (tagsArray2[i] == tagsArray[ii])
					 {
						 tagCase = 1;
					 }
				}
				if (tagCase != 1 && tagsArray2[i])
				{
					newTags += tagsArray2[i] + ", ";
				}
		   }

           if (newTags == "") return;
		   newTags = newTag + ", " + newTags;
		   newTags = newTags.substr(0, newTags.length-2);
		   tab.vars.Set('DOpusTag', newTags);

		   // Write new tags to DOpusTag.dat
		   var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
		   var oFileWrite = fsu.OpenFile(tagFile, "wa");
           var utf8bom = ST.Encode(newTags, "utf-8 bom");   // Decode as UTF-8 with BOM
           if (oFileWrite.error == 0)
           {
               oFileWrite.Write(utf8bom)
           }
              oFileWrite.Close()
	   }
	} while (msg == true)
}



function DeleteTag() {

    var deleteTag = tab.vars.Get('deleteTag');
	if (deleteTag == "") return;

	var tags = tab.vars.Get('DOpusTag');
	var tagsArray = tags.split(", ");                           // Seprator
	var newTags = "";
	for (i = 0; tagsArray[i]; i++)
	{
	     if (tagsArray[i] != deleteTag)
		 {
			 newTags += tagsArray[i] + ", ";
         }
	}

    newTags = newTags.substr(0, newTags.length-2)
    tab.vars.Set('DOpusTag', newTags);

    // Write new tags to DOpusTag.dat
	var tagFile = "%appdata%\\GPSoftware\\Directory Opus\\Scripts\\INI\\DOpusTag.dat";   // File path
	var oFileWrite = fsu.OpenFile(tagFile, "wa");
    var utf8bom = ST.Encode(newTags, "utf-8 bom");   // Decode as UTF-8 with BOM
    if (oFileWrite.error == 0)
    {
        oFileWrite.Write(utf8bom)
    }
        oFileWrite.Close()
}



function format_tags(str){

    var array = String(str).split(",");                                // Seprator
	var newTag1 = array[0].trim().replace(/\|/g,'');
	if (newTag1 == "") return newTags = "";

    var tag, newTags = "";
	for (i = 0; array[i]; i++)
	{
		 tag = array[i].trim().replace(/\|/g,'');
		 if (tag != "")
		 {
		     newTags += tag + ", ";
		 }
    }
	newTags = newTags.substr(0, newTags.length-2);

    var array2 = newTags.split("\r\n");                                // Seprator
	newTags = "";
	for (i = 0; array2[i]; i++)
	{
		 tag = array2[i].trim().replace(/\|/g,'');
		 if (tag != "")
		 {
		     newTags += tag + ", ";
		 }
	}
	newTags = newTags.substr(0, newTags.length-2);
	return newTags;
}



function remove_duplicate(str) {

    var array = String(str).split(", ");                               // Seprator
	var newTag1 = array[0].trim().replace(/\|/g,'');
	if (newTag1 == "") {return newTag1};

    var vecTags = DOpus.NewVector();
    for (i=0; array[i]; i++)
    {
	     vecTags.push_back = array[i];
    }

    var removedNum = vecTags.unique();
    if (removedNum == 0) {return String(str)};

    var newTags = "";
    for (var eItems = new Enumerator(vecTags); !eItems.atEnd(); eItems.moveNext() )
    {
	     newTags += eItems.item() + ", ";
    }
    newTags = newTags.substr(0, newTags.length-2);
	return newTags;
}



==SCRIPT RESOURCES
<resources>
	<resource name="dlgCombox" type="dialog">
		<dialog fontsize="8" height="88" lang="english" resize="yes" title="Directory Opus" width="322">
			<control edit="yes" height="40" name="combo" type="combo" width="306" x="8" y="50" />
			<control close="3" height="14" name="btn4" resize="xy" title="&amp;Here" type="button" width="60" x="81" y="67" />
			<control close="2" height="14" name="btn3" resize="xy" title="&amp;Below" type="button" width="60" x="145" y="67" />
			<control halign="left" height="8" name="static" title="Topic" type="static" valign="top" width="280" x="30" y="10" />
			<control halign="left" height="16" name="static2" title="Notes" type="static" valign="top" width="280" x="30" y="22" />
			<control halign="center" height="14" image="yes" name="static3" title="Icon" type="static" valign="top" width="14" x="8" y="7" />
			<control close="1" default="yes" height="14" name="btnOK" resize="xy" title="&amp;OK" type="button" width="50" x="209" y="67" />
			<control close="0" height="14" name="btnCancel" resize="xy" title="&amp;Cancel" type="button" width="50" x="263" y="67" />
		</dialog>
	</resource>
	<resource name="dlgEditTags" type="dialog">
		<dialog fontsize="8" height="120" lang="english" resize="yes" title="Edit Tags" width="522">
			<control halign="left" height="62" multiline="yes" name="editTags" resize="wh" type="edit" width="506" x="8" y="32" />
			<control close="1" default="yes" height="14" name="btnApply" resize="xy" title="&amp;Apply" type="button" width="50" x="409" y="99" />
			<control close="0" height="14" name="btnCancel" resize="xy" title="&amp;Cancel" type="button" width="50" x="462" y="99" />
			<control halign="left" height="8" name="static" title="Edit Tags" type="static" valign="top" width="472" x="39" y="7" />
			<control halign="left" height="20" image="yes" name="static3" title="Static" type="static" valign="top" width="20" x="10" y="4" />
			<control halign="left" height="8" name="static2" title="Notes" type="static" valign="top" width="472" x="39" y="17" />
		</dialog>
	</resource>
	<resource name="dlgList" type="dialog">
		<dialog fontsize="9" height="350" lang="english" resize="yes" title="Directory Opus" width="279">
			<control halign="left" height="12" name="editFilter" resize="xy" tip="Filter" type="edit" width="118" x="21" y="331" />
			<control checkboxes="auto" fullrow="yes" height="287" name="list" noheader="yes" resize="wh" type="listview" viewmode="details" width="266" x="6" y="40" />
			<control close="1" default="yes" height="12" name="btnOK" resize="xy" title="&amp;OK" type="button" width="42" x="184" y="332" />
			<control close="0" height="12" name="btnClose" resize="xy" title="C&amp;ancel" type="button" width="42" x="229" y="332" />
			<control halign="left" height="8" name="static" title="Topic" type="static" valign="top" width="245" x="25" y="7" />
			<control halign="left" height="13" image="yes" name="static3" title="Static" type="static" valign="top" width="13" x="7" y="7" />
			<control halign="left" height="12" image="yes" name="static4" resize="xy" title="Static" type="static" valign="top" width="12" x="8" y="331" />
			<control halign="left" height="16" name="static2" title="Notes" type="static" valign="top" width="245" x="25" y="20" />
			<control halign="left" height="5" name="static5" title="Static" type="static" valign="top" width="6" x="130" y="335" />
		</dialog>
	</resource>
</resources>